/*Scram16k_sw.c Voice scrambler with upsampling and user-dip switches*/

#include "c6xdsk.h"
#include "sine8k.h"
#include "sine16k.h"
#include "coeff8k.h"
#include "coeff16k.h"
short processdata16k(short data);
short reprocessdata16k(short data);
short filter16k(short inp,short *mem);
short MultSine16k1(short input);
short MultSine16k2(short input);
short processdata8k(short data);
short reprocessdata8k(short data);
short filter8k(short inp,short *mem);
short MultSine8k1(short input);
short MultSine8k2(short input);
unsigned int get_ioport(void);
static	short filter16k1[COEFF16K],filter16k2[COEFF16K];
static	short filter8k1[COEFF8K],filter8k2[COEFF8K];
static	short filter16k3[COEFF16K],filter16k4[COEFF16K];
static	short filter8k3[COEFF8K],filter8k4[COEFF8K];
short input, output;
int i,user_dip_settings,io_port_values;

void main()
{
	short temp1,temp2;
  	comm_poll();
  for( i=0; i< COEFF16K; i++)
  {
	filter16k1[i] = 0;
   filter16k2[i] = 0;
  }
  while(1)
  {
   input = input_sample();

  *(unsigned volatile int *)EMIF_CE1 = CE1_32; // EMIF CE1 control, 32bit async
  io_port_values=get_ioport();
  user_dip_settings=io_port_values&0x7000000;
  *(unsigned volatile int *)EMIF_CE1 = CE1_8; // EMIF CE1 control, 8bit async

if ((user_dip_settings&0x7000000) == 0x04000000)
	{
	processdata16k(input);
	output = processdata16k(input);
  	}
else if ((user_dip_settings&0x7000000) == 0x02000000)
	{
	output = processdata8k(input);
  	}
else if ((user_dip_settings&0x7000000) == 0x01000000)
	{
	output = input;
	}
else if ((user_dip_settings&0x7000000) == 0x05000000)
	{
	temp1 = processdata16k(input);
	temp2 = processdata16k(input);
	reprocessdata16k(temp1);
	output = reprocessdata16k(temp1);
  	}
else if ((user_dip_settings&0x7000000) == 0x03000000)
	{
	output = processdata8k(input);
	output = reprocessdata8k(output);
  	}
else if ((user_dip_settings&0x7000000) == 0)
	{
	output = filter8k(input,filter8k3);
  	}
else if ((user_dip_settings&0x7000000) == 0x07000000)
	{
	filter16k(input,filter16k3);
	output = filter16k(input,filter16k3);
  	}
   output_sample(output);
  }
}

short processdata16k(short data)
{
	data = filter16k(data,filter16k1);
   data = MultSine16k1(data);
	data = filter16k(data,filter16k2);
return data;
}

short reprocessdata16k(short data)
{
	data = filter16k(data,filter16k3);
   data = MultSine16k2(data);
	data = filter16k(data,filter16k4);
return data;
}

short filter16k(short inp,short *mem)
{
	int j;
	long acc;
		mem[COEFF16K-1] = inp;
		acc = mem[0] * coeff16k[0];

		for (j = 1; j < COEFF16K; j++)
		{
			acc += mem[j] * coeff16k[j];
			mem[j-1] = mem[j];
		}
		acc = (acc)>>15;
		return acc;
}

short MultSine16k1(short input)
{
    static int j=0;

  	input = (input * sine16k[j++]) >> 11;
    if(j>=SINE16K) j = 0;

    return input;
}

short MultSine16k2(short input)
{
    static int j=0;

  	input = (input * sine16k[j++]) >> 11;
    if(j>=SINE16K) j = 0;

    return input;
}

short processdata8k(short data)
{
	data = filter8k(data,filter8k1);
   data = MultSine8k1(data);
	data = filter8k(data,filter8k2);
return data;
}

short reprocessdata8k(short data)
{
	data = filter8k(data,filter8k3);
   data = MultSine8k2(data);
	data = filter8k(data,filter8k4);
return data;
}

short filter8k(short inp,short *mem)
{
	int j;
	long acc;

		mem[COEFF8K-1] = inp;
		acc = mem[0] * coeff8k[0];

		for (j = 1; j < COEFF8K; j++)
		{
			acc += mem[j] * coeff8k[j];
			mem[j-1] = mem[j];
		}
		acc = ((acc)>>15);
		return acc;
}

short MultSine8k1(short input)
{
    static int j=0;

  	input = (input * sine8k[j++]) >> 11;
    if(j>=SINE8K) j = 0;

    return input;
}

short MultSine8k2(short input)
{
    static int j=0;

  	input = (input * sine8k[j++]) >> 11;
    if(j>=SINE8K) j = 0;

    return input;
}



/*-------------------------------------------------------------------------*/
/* get_ioport() - used to read state of ioport                             */
/*-------------------------------------------------------------------------*/
unsigned int get_ioport(void)
{
  return(*(unsigned volatile int *)IO_PORT);
}
